﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace ComplusInstaller
{
    class Program
    {
        static void Main(string[] args)
        {
            string appName = "ClassLibraryAsAdmin";
            string folderPath = Path.GetDirectoryName(typeof(Program).Assembly.Location);
            string complusPath = Path.Combine(folderPath, appName + ".dll");

            Guid guid = new Guid("{F618C514-DFB8-11D1-A2CF-00805FC79235}");
            Type type = Type.GetTypeFromCLSID(guid);

            object comObject = Activator.CreateInstance(type);

            ICOMAdminCatalog2 catalog = comObject as ICOMAdminCatalog2;

            ICatalogCollection apps = catalog.GetCollection("Applications") as ICatalogCollection;
            apps.Populate();

            // 삭제하고,
            //DeleteService(catalog, apps, appName);
            //DeleteComplus(catalog, apps, appName);

            //// 다시 설치
            //InstallComplusApplication(catalog, apps, appName);
            //InstallComplusComponent(catalog, apps, appName, complusPath);

            // NT 서비스로 설치
            InstallService(catalog, apps, appName);
        }

        private static void InstallService(ICOMAdminCatalog2 catalog, ICatalogCollection apps, string appName)
        {
            catalog.CreateServiceForApplication(appName, appName, "SERVICE_DEMAND_START", "SERVICE_ERROR_NORMAL",
                string.Empty, @".\LOCALSYSTEM", null, false);
        }

        private static void InstallComplusComponent(ICOMAdminCatalog2 catalog, ICatalogCollection apps, string appName, string modulePath)
        {
            string dllPath = Path.ChangeExtension(modulePath, ".dll");
            catalog.InstallComponent(appName, dllPath, string.Empty, string.Empty);
        }

        private static void InstallComplusApplication(ICOMAdminCatalog2 catalog, ICatalogCollection apps, string appName)
        {
            ICatalogObject catObject = apps.Add() as ICatalogObject;
            catObject["Name"] = appName;
            catObject["Activation"] = COMAdminActivationOptions.COMAdminActivationLocal;
            catObject["ApplicationAccessChecksEnabled"] = false;
            catObject["ShutdownAfter"] = 1;
            catObject["Identity"] = @"nt authority\system";
            apps.SaveChanges();
        }

        private static void DeleteComplus(ICOMAdminCatalog2 catalog, ICatalogCollection apps, string appName)
        {
            int appIndex = HasApplication(apps, appName);
            if (appIndex == -1)
            {
                return;
            }

            apps.Remove(appIndex);
            apps.SaveChanges();
        }

        private static int HasApplication(ICatalogCollection apps, string appName)
        {
            int appCount = apps.Count;
            string targetAppName = appName.ToLower();

            for (int i = 0; i < appCount; i++)
            {
                ICatalogObject app = apps[i] as ICatalogObject;
                if (((string)app.Name).ToLower() == targetAppName)
                {
                    return i;
                }
            }

            return -1;
        }

        private static void DeleteService(ICOMAdminCatalog2 catalog, ICatalogCollection apps, string appName)
        {
            if (HasApplication(apps, appName) == -1)
            {
                return;
            }

            try
            {
                catalog.ShutdownApplication(appName);
                catalog.DeleteServiceForApplication(appName);
            }
            catch { }
        }
    }
}
