﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Diagnostics;
using System.Text;

namespace WebApplication1
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            AdminCode adminCode = new AdminCode();
            int id = Process.GetCurrentProcess().Id;

            StringBuilder sb = new StringBuilder();

            foreach (var item in adminCode.GetCounters(id))
            {
                sb.AppendLine(item.Key + " == " + item.Value.ToString() + "<br />");
            }

            this.Label1.Text = sb.ToString();
        }
    }

    public class AdminCode
    {
        public AdminCode()
        {
        }

        public Dictionary<string, long> GetCounters(int pid)
        {
            string name = GetProcessInstanceName(pid);
            List<PerformanceCounter> list = new List<PerformanceCounter>();

            list.Add(new PerformanceCounter(".NET CLR Memory", "Gen 1 heap Size", name));
            list.Add(new PerformanceCounter(".NET CLR Memory", "Gen 2 heap Size", name));
            list.Add(new PerformanceCounter(".NET CLR Memory", "Large Object Heap size", name));
            list.Add(new PerformanceCounter(".NET CLR Memory", "Finalization Survivors", name));
            list.Add(new PerformanceCounter(".NET CLR Memory", "% Time in GC", name));
            list.Add(new PerformanceCounter(".NET CLR Memory", "# of Sink Blocks in use", name));

            Dictionary<string, long> result = new Dictionary<string, long>();

            foreach (var item in list)
            {
                result.Add(item.CounterName, GetCounterValue(item));
            }

            return result;
        }

        private static long GetCounterValue(PerformanceCounter item)
        {
            switch (item.CounterType)
            {
                case PerformanceCounterType.NumberOfItems32:
                case PerformanceCounterType.NumberOfItems64:
                    return item.RawValue;

                case PerformanceCounterType.RateOfCountsPerSecond32:
                case PerformanceCounterType.RateOfCountsPerSecond64:
                    return (long)item.NextValue();

                case PerformanceCounterType.AverageTimer32:
                    return (long)item.NextValue();
            }

            return 0;
        }

        private static string GetProcessInstanceName(int pid)
        {
            //foreach (var item in PerformanceCounterCategory.GetCategories())
            //{
            //    if (item.CategoryName.StartsWith(".") == true)
            //    {
            //        Console.WriteLine(item.CategoryName);
            //        System.Diagnostics.Trace.WriteLine(item.CategoryName);
            //    }
            //}

            PerformanceCounterCategory cat = new PerformanceCounterCategory("Process");

            string[] instances = cat.GetInstanceNames();
            foreach (string instance in instances)
            {
                using (PerformanceCounter cnt = new PerformanceCounter("Process",
                     "ID Process", instance, true))
                {
                    int val = (int)cnt.RawValue;
                    if (val == pid)
                    {
                        return instance;
                    }
                }
            }
            throw new Exception("Could not find performance counter " +
                "instance name for current process. This is truly strange ...");
        }
    }

}